# -*- coding: utf8 -*-
from __future__ import unicode_literals

import csv
import datetime
import logging
import six
from six import BytesIO
from six import StringIO

from driverng import MessageV2, Worklist
from driverng.splitted import Splitted, String

logger = logging.getLogger("driverng.cfx_biorad")


class CfxBioradBase(Splitted):

    SEPARATOR = ";"


class CfxBioradResult(object):
    FIELDS = [
        String("well"),
        String("fluor"),
        String("target"),
        String("content"),
        String("sample"),
        String("cq"),
    ]

    def parse(self, line):
        fields = self.FIELDS
        for index, field in enumerate(fields):
            setattr(self, field.name, line[index])


class CfxBioradResultsMultiple(list):
    _MESSAGE_TYPE_MAP = {MessageV2.RESULT: CfxBioradResult}

    def add(self, type_message=None, **kwargs):
        obj = self._MESSAGE_TYPE_MAP[type_message](**kwargs)
        self.append(obj)
        return obj

    def parse(self, data):
        if six.PY2:
            buf = BytesIO(data.encode("utf-8"))
        else:
            buf = StringIO(data)
        reader = csv.reader(buf, delimiter=str(","))
        self.date_resu = None
        for row in reader:
            if six.PY2:
                line = [
                    c.decode("utf-8") for c in row
                ]  # PY2 csv reader est incompatible avec unicode
            else:
                line = row
            if not len(line):
                continue
            if line[0] == "Run Ended":
                try:
                    self.date_resu = datetime.datetime.strptime(
                        line[1].strip()[:19], "%m/%d/%Y %H:%M:%S"
                    )
                except ValueError as e:
                    logger.info("La date des résultat n'est pas au bon format %s", e)
            if len(line) == 6:
                obj = self.add(type_message=MessageV2.RESULT)
                obj.parse(line)


class Driver(MessageV2):
    SPEC = {"code": "cfx_biorad", "label": "Cfx Bio-Rad CSV"}

    def parse(self, data):
        obj = CfxBioradResultsMultiple()
        obj.parse(data)
        return obj

    def decode_message(self, message):
        worklist = Worklist(context="result")
        instrument_serial_number = None
        for m in message:
            if m.well == "Base Serial Number":
                instrument_serial_number = m.fluor
                continue
            elif not m.target and not m.sample and not m.cq:
                continue
            elif m.sample == "Sample" and m.well == "Well" and m.cq == "Cq":
                continue
            sample = worklist.get_or_create(
                sample_id=m.sample, rack="CFX-{}".format(m.well), type="patient"
            )
            sample.results.add(
                code=m.target, value=m.cq, instrument=instrument_serial_number
            )
            sample.results.add(
                code="{}{}".format(m.fluor, m.target),
                value=m.cq,
                instrument=instrument_serial_number,
                date=message.date_resu,
            )
        return worklist
